#include "opencvviewer.h"

using namespace cv;

OpenCVViewer::OpenCVViewer(QWidget *parent)
    : QGraphicsView(parent)
{
    scene = new QGraphicsScene(this);
    this->setScene(scene);
}

OpenCVViewer::~OpenCVViewer()
{
    delete scene;
}

QSize OpenCVViewer::sizeHint() const
{
    return QSize(400, 300);
}

Mat OpenCVViewer::getImage()
{
    return displayImage;
}

void OpenCVViewer::setImage(Mat newImage)
{
    if(newImage.empty()) {
        return;
    }

    if(newImage.channels() == 3 || newImage.channels() == 4) {
        cvtColor(newImage, displayImage, COLOR_BGR2RGB); // Copy and convert the new image.
    } else {
        cvtColor(newImage, displayImage, COLOR_GRAY2RGB); // Copy and convert the new image.
    }

    QImage img((uchar*)displayImage.data, displayImage.cols, displayImage.rows, displayImage.step, QImage::Format_RGB888);

    scene->clear();
    scene->addPixmap(QPixmap::fromImage(img));
}

void OpenCVViewer::resizeEvent(QResizeEvent* event)
{
    this->fitInView(scene->sceneRect(), Qt::KeepAspectRatio);
}

void OpenCVViewer::mousePressEvent(QMouseEvent* event)
{
    emit mouseClicked(event);
}

void OpenCVViewer::mouseMoveEvent(QMouseEvent* event)
{
    emit mouseMoved(event);
}

void OpenCVViewer::mouseReleaseEvent(QMouseEvent* event)
{
    emit mouseReleased(event);
}

void OpenCVViewer::wheelEvent(QWheelEvent* event)
{
    emit wheelScrolled(event);
}

cv::Point OpenCVViewer::mapPoint(cv::Point widgetCoords)
{
    if(displayImage.empty()) { // Bail out if we have no image to map to.
        return cv::Point(0,0);
    }

    QPointF mapResult = this->mapToScene(widgetCoords.x, widgetCoords.y);
    return cv::Point((int)cvUtils::round(mapResult.x()), (int)cvUtils::round(mapResult.y()));
}
