#include "pixelannotation.h"

PixelAnnotation::PixelAnnotation(int id, QStringList metaDataNames, QStringList metaDataTypes)
{
    assert(metaDataNames.size() == metaDataTypes.size());

    this->theId = id;

    for(int i = 0; i < (int)metaDataNames.size(); i++) {
        if(metaDataTypes[i] == "bool") {
            metaDataValues.push_back("false");
        } else {
            metaDataValues.push_back(QString());
        }
    }
    this->metaDataNames = metaDataNames;
    this->metaDataTypes = metaDataTypes;
}

int PixelAnnotation::id()
{
    return theId;
}

void PixelAnnotation::id(int id)
{
    theId = id;
}

cv::Mat PixelAnnotation::getMask()
{
    // Return the combined mask and don't care mask
    cv::Mat returnMask = maskNoDontCare.clone();

    if (!maskOnlyDontCare.empty()) {
        returnMask = returnMask + maskNoDontCare;
    }

    return returnMask;
}

cv::Mat PixelAnnotation::getMaskNoDontCare()
{
    return maskNoDontCare;
}

cv::Mat PixelAnnotation::getMaskOnlyDontCare()
{
    return maskOnlyDontCare;
}

void PixelAnnotation::setMask(cv::Mat mask, bool addDontCare, int borderThickness, int borderColor)
{
    maskNoDontCare = mask.clone();

    // Update the don't care mask if needed
    if (addDontCare) {
        cv::Mat input = (mask.clone() == cv::GC_FGD) + (mask.clone() == cv::GC_PR_FGD);
        maskOnlyDontCare = cv::Mat::zeros(mask.size(), mask.type());

        generateDontCareFromMask(input, maskOnlyDontCare, borderThickness, borderColor);
    }
}

void PixelAnnotation::generateDontCareFromMask(cv::Mat mask, cv::Mat &onlyDontCare, int borderThickness, int borderColor)
{
    cv::Mat input = mask.clone();
    if (countNonZero(input) == 0) {
        return;
    }

    std::vector<std::vector<cv::Point> > contours;
    std::vector<cv::Vec4i> hierarchy;
    onlyDontCare = cv::Mat::zeros(mask.size(), mask.type());

    cv::findContours(input, contours, hierarchy, CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE);
    
    if (!hierarchy.empty()) {
        for (int i = 0; i >= 0; i = hierarchy[i][0]) {
            drawContours(onlyDontCare, contours, i, cv::Scalar(borderColor), borderThickness, 8, hierarchy);
        }
    }
}


void PixelAnnotation::setDontCareMask(cv::Mat dontCareMask)
{
    maskOnlyDontCare = dontCareMask.clone();
}

int PixelAnnotation::getMetaDataSize()
{
    return metaDataValues.size();
}

QStringList PixelAnnotation::getMetaData()
{
    return metaDataValues;
}

QString PixelAnnotation::getMetaData(int metaDataId)
{
    return metaDataValues[metaDataId];
}

void PixelAnnotation::setMetaData(int metaDataId, QString value)
{
    assert(metaDataTypes[metaDataId] == "string" || (metaDataTypes[metaDataId] == "bool" && (value == "false" || value == "true")));

    metaDataValues[metaDataId] = value;
}

void PixelAnnotation::setMetaData(int metaDataId, bool value)
{
    assert(metaDataTypes[metaDataId] == "bool");
    if(value) {
        metaDataValues[metaDataId] = "true";
    } else {
        metaDataValues[metaDataId] = "false";
    }
}

void PixelAnnotation::setMetaData(QStringList values)
{
    assert(values.size() == metaDataValues.size());
    metaDataValues = values;
}

void PixelAnnotation::toggleMetaData(int metaDataId)
{
    if(metaDataTypes[metaDataId] == "bool") {
        if(metaDataValues[metaDataId] == "false") {
            metaDataValues[metaDataId] = "true";
        } else {
            metaDataValues[metaDataId] = "false";
        }
    }
}

QStringList PixelAnnotation::getMetaDataNames()
{
    return metaDataNames;
}

QString PixelAnnotation::getMetaDataName(int metaDataId)
{
    return metaDataNames[metaDataId];
}

QString PixelAnnotation::getMetaDataType(int metaDataId)
{
    return metaDataTypes[metaDataId];
}
